package javax.slee.profile;

import javax.slee.Address;

/**
 * This interface is implemented by profile updated events generated by
 * <code>ProfileMBean</code> objects when a profile is updated.
 * <p>
 * The event type name of profile updated events is
 * "<code>javax.slee.profile.ProfileUpdatedEvent</code>".
 */
public interface ProfileUpdatedEvent {
    /**
     * Get the profile identifer for the profile that was updated.
     * @return the profile identifier for the profile that was updated.
     */
    public ProfileID getProfile();

    /**
     * Get the address of the profile that was updated.  This method is equivalent
     * to {@link ProfileID#toAddress() this.getProfile().toAddress()}.
     * @return the address of the profile that was updated.
     */
    public Address getProfileAddress();

    /**
     * Get the committed state of the profile before it was updated.  The actual type of the
     * return value is the Profile CMP Interface defined by the profile specification
     * of the profile table in which the profile resides, and may be safely typecast
     * using the normal Java typecast mechanism.
     * <p>
     * The object returned from this method is only valid within the event-handler transaction
     * in which the event was delivered to an SBB entity.
     * @return the committed state of the profile before it was updated.
     */
    public Object getBeforeUpdateProfile();

    /**
     * Get the committed state of the profile after it was updated.  The actual type of the
     * return value is the Profile CMP Interface defined by the profile specification
     * of the profile table in which the profile resides, and may be safely typecast
     * using the normal Java typecast mechanism.
     * <p>
     * The object returned from this method is only valid within the event-handler transaction
     * in which the event was delivered to an SBB entity.
     * @return the committed state of the profile after it was updated.
     */
    public Object getAfterUpdateProfile();
}

